@@ -63,29 +63,22 @@ class SubscriptionManager:
             await self._init_web_socket()
         except Exception:  # pylint: disable=broad-except
             _LOGGER.debug("Failed to connect. Reconnecting... ", exc_info=True)
-            self._state = STATE_STOPPED
             self.retry()
             return
 
         _LOGGER.debug("Running")
         self._state = STATE_RUNNING
-
         try:
             await self._running_loop()
         except Exception:  # pylint: disable=broad-except
-            if self._state != STATE_STOPPED:
-                _LOGGER.error("Connection error", exc_info=True)
-            else:
-                _LOGGER.debug("Connection error", exc_info=True)
-        finally:
-            _LOGGER.debug("Closing running task.")
             if self._state == STATE_STOPPED:
-                return
-            _LOGGER.debug("Reconnecting")
-            self._state = STATE_STOPPED
-            self.retry()
+                _LOGGER.debug("Connection error", exc_info=True)
+                self.retry()
+            else:
+                _LOGGER.error("Connection error", exc_info=True)
 
-    async def stop(self, timeout=10):
+
+    async def stop(self):
         """Close websocket connection."""
         _LOGGER.debug("Stopping client.")
         start_time = time()
@@ -95,36 +88,15 @@ class SubscriptionManager:
             _LOGGER.debug("Sending unsubscribe: %s", subscription_id)
             await self.unsubscribe(subscription_id)
 
-        while (
-            timeout > 0
-            and self.websocket is not None
-            and not self.subscriptions
-            and (time() - start_time) < timeout / 2
-        ):
-            await asyncio.sleep(0.1)
-
         self._state = STATE_STOPPED
         await self._close_websocket()
 
-        while (
-            timeout > 0
-            and self.websocket is not None
-            and not self.websocket.closed
-            and (time() - start_time) < timeout
-        ):
-
-            await asyncio.sleep(0.1)
-
         self._cancel_client_task()
         _LOGGER.debug("Server connection is stopped")
 
     def retry(self):
         """Retry to connect to websocket."""
         _LOGGER.debug("Retry, state: %s", self._state)
-        if self._state in [STATE_STARTING, STATE_RUNNING]:
-            _LOGGER.debug("Skip retry since state: %s", self._state)
-            return
-        _LOGGER.debug("Cancel retry timer")
         self._cancel_retry_timer()
         self._state = STATE_STARTING
         _LOGGER.debug("Restart")
@@ -173,7 +145,7 @@ class SubscriptionManager:
         finally:
             self.websocket = None
 
-    async def _process_msg(self, msg):
+    def _process_msg(self, msg):
         """Process received msg."""
         result = json.loads(msg)
 
@@ -237,11 +209,11 @@ class SubscriptionManager:
         k = 0
         while self.is_running:
             try:
-                msg = await asyncio.wait_for(self.websocket.recv(), timeout=10)
+                msg = await asyncio.wait_for(self.websocket.recv(), timeout=30)
             except asyncio.TimeoutError:
-                if k > 180:
+                k += 1
+                if k > 5:
                     _LOGGER.debug("No data, reconnecting.")
-                    self._state = STATE_STOPPED
                     self.retry()
                     return
 
@@ -250,13 +222,9 @@ class SubscriptionManager:
                     pong_waiter = await self.websocket.ping()
                     await asyncio.wait_for(pong_waiter, timeout=3)
                 except asyncio.TimeoutError:
-                    _LOGGER.error("No response to ping in 3 seconds, reconnecting.")
-                    _LOGGER.debug("Reconnecting")
-                    self._state = STATE_STOPPED
+                    _LOGGER.error("No response to ping, reconnecting.")
                     self.retry()
 
-                k += 1
-
             else:
                 k = 0
-                await self._process_msg(msg)
+                self._process_msg(msg)
